#!/usr/bin/perl -w
# convert image to binary and/or assembler file
# (C) Arne Rossius 2012
#
# 2012-09-09  first version

$hor = 0; # horizontal bytes?
$rev = 1; # set to 1 for MSB-left or MSB-top
$bin = 0; # enable binary file output
$asm = 1; # enable assembler file output
$c   = 0; # enable C file output
$header = 0; # add header (2 bytes width, 2 bytes height)

use Image::Magick;

if (! @ARGV) {
	die ("Usage: $0 <Filename.png> [<Filename.png> [...]]\n");
}

foreach $file (@ARGV) {
	if (! -f $file) {
		die ("Image file '$file' not found!\n");
	}
	
	# read image file
	$image = Image::Magick->new;
	$image->Read($file);
	$width = $image->Get('width');
	$height = $image->Get('height');
	print "Converting '$file' ($width x $height) ...\n";
	@pixels = $image->GetPixels(map=>'I', width=>$width, height=>$height, normalize=>1);
	
	# open output file(s)
	$filename = $file;
	$filename =~ s/\.[A-Za-z0-9]+$//;
	if ($c) {
		open C, ">$filename.c" or die ("Failed to open C file.\n");
	}
	if ($asm) {
		open ASM, ">$filename.asm" or die ("Failed to open ASM file.\n");
	}
	if ($bin) {
		open BIN, ">$filename.bin" or die ("Failed to open BIN file.\n");
	}
	
	# output header
	if ($c) {
		$var = $filename;
		$var =~ s/^[0-9]|[^a-zA-Z0-9]/_/g;
		print C "#include <stdint.h>\n";
		print C "#include <avr/pgmspace.h>\n";
		print C "PROGMEM uint8_t ${var}[] = {\n";
	}
	if ($header) {
		if ($c) {
			printf C "\t%02x, %02x, %02x, %02x,\n", $width & 0xFF, ($width >> 8) & 0xFF, $height & 0xFF, ($height >> 8) & 0xFF;
		}
		if ($asm) {
			print ASM ".dw $width, $height"; # missing \n is intentional
		}
		if ($bin) {
			print BIN pack('n', $width), pack('n', $height);
		}
	}
	
	# convert image
	if ($hor) {
		$xinc = 8;
		$yinc = 1;
	} else {
		$xinc = 1;
		$yinc = 8;
	}
	$n = 0;
	for ($y = 0; $y < $height; $y += $yinc) {
		for ($x = 0; $x < $width; $x += $xinc) {
			if ($c) {
				if ($n % 8 == 0) {
					if ($y + $x > 0) {
						print C ",\n\t";
					} else {
						print C "\t";
					}
				} else {
					print C ", ";
				}
			}
			if ($asm) {
				if ($n % 8 == 0) {
					print ASM "\n.db ";
				} else {
					print ASM ", ";
				}
			}
			$data = 0;
			for ($bit = 0; $bit < 8; $bit++) {
				if ($hor) {
					last if ($x + $bit >= $width);
					$cx = $x + $bit;
					$cy = $y;
				} else {
					last if ($y + $bit >= $height);
					$cx = $x;
					$cy = $y + $bit;
				}
				if ($pixels[$cy * $width + $cx] < .5) {
					if ($rev) {
						$data |= 1<<(7-$bit);
					} else {
						$data |= 1<<$bit;
					}
				}
			}
			if ($c) {
				printf C "0x%02x", $data;
			}
			if ($asm) {
				printf ASM "0x%02x", $data;
			}
			if ($bin) {
				print BIN pack('C', $data);
			}
			$n++;
		}
	}
	
	# close files
	if ($c) {
		print C "\n};\n";
		close C;
	}
	if ($asm) {
		print ASM "\n";
		close ASM;
	}
	if ($bin) {
		close BIN;
	}
}
