;Test Program for PCF8548 LCD, for Pollin VG-090651 LCD
;ATtiny2313 @ 1 MHz (internal R/C oscillator), lfuse = 0x44, hfuse=0xDB
;
;PORTS
;  PB0 = I2C SDA (external pull-up 4.7k)
;  PB1 = I2C SCL (external pull-up 4.7k)
;  PB2 = LCD Reset (active low)
;
;VERSIONS
;  2014-08-22  Arne Rossius
;   * first version

.include "../include/tn2313def.inc"

.def	temp = R16
.def	temp2 = R17
.def	lcd_x = R18
.def	lcd_y = R19

.equ	LCD_PORT = PORTB
.equ	LCD_PIN = PINB
.equ	LCD_DDR = DDRB
.equ	LCD_SDA = 0
.equ	LCD_SCL = 1
.equ	LCD_RESET = 2

;===============================================================================

.cseg
.org 0x000
	rjmp	reset
	
image:
	.include "images/geese.asm"
	
;===============================================================================

reset:
	;set stackpointer
	ldi	temp, RAMEND
	out	SPL, temp
	
	;init ports
	ldi	temp, 0x03
	out	DDRA, temp
	ldi	temp, 0x7F
	out	DDRD, temp
	ldi	temp, 0xFC
	out	DDRB, temp
	ldi	temp, 0x04
	out	PORTB, temp
	
	;delay
	clr	temp
	clr	temp2
	dec	temp2
	brne	PC-1
	dec	temp
	brne	PC-3
	
	;init LCD
	rcall	lcd_init
	
	;display image
	ldi	ZH, HIGH(image*2)
	ldi	ZL, LOW(image*2)
	ldi	lcd_y, 0
image_loop_y:
	rcall	i2c_start
	ldi	temp, 0x78 ;address
	rcall	i2c_send
	ldi	temp, 0x80 ;command + new control byte follows
	rcall	i2c_send
	ldi	temp, 0x80 ;set x address
	rcall	i2c_send
	ldi	temp, 0x80 ;command + new control byte follows
	rcall	i2c_send
	ldi	temp, 0x40 ;set y address
	add	temp, lcd_y
	rcall	i2c_send
	ldi	temp, 0x40 ;data follows
	rcall	i2c_send
	ldi	lcd_x, 0
image_loop_x:
	lpm	temp, Z+
	rcall	i2c_send
	inc	lcd_x
	cpi	lcd_x, 96 ;LCD width: 95 px
	brlo	image_loop_x
	rcall	i2c_stop
	inc	lcd_y
	cpi	lcd_y, 8 ;LCD height: 60 px => 8 bytes
	brlo	image_loop_y
	
;-------------------------------------------------------------------------------

loop:
	rjmp	loop
	
;===============================================================================

lcd_init:
	;assert #RESET
	cbi	LCD_PORT, LCD_RESET
	;delay
	clr	temp
	dec	temp
	brne	PC-1
	;de-assert #RESET
	sbi	LCD_PORT, LCD_RESET
	;delay
	clr	temp
	dec	temp
	brne	PC-1
	
	;send init commands
	rcall	i2c_start
	ldi	temp, 0x78 ;Address
	rcall	i2c_send
	ldi	temp, 0x00 ;Control byte: last control byte, commands follow
	rcall	i2c_send
	
	ldi	temp, 0x21 ;Function Set: Mirroring off, Power on, Increment x,
	rcall	i2c_send   ;              Select extended instruction set (H=1)
	ldi	temp, 0x04 ;Temperature Coefficient: 0
	rcall	i2c_send
	ldi	temp, 0x08 ;Display Config: Top = MSB, Mirroring off
	rcall	i2c_send
;	ldi	temp, 0x16 ;Bias System: Bias = 1/5
	ldi	temp, 0x12 ;Bias System: Bias = 1/9 (recommended for 1/65 Mux)
	rcall	i2c_send
;	ldi	temp, 0x80 + 96 ;Set V_OP: 96/127 => 5.82V (with PRS=0)
	ldi	temp, 0x80 + 58 ;Set V_OP: 58/127 => 8.49V (with PRS=1)
	rcall	i2c_send
	ldi	temp, 0x20 ;Function Set: Mirroring off, Power on, Increment x,
	rcall	i2c_send   ;              Select standard instruction set (H=0)
;	ldi	temp, 0x04 ;Set V_LCD Range: lower range (PRS=0)
	ldi	temp, 0x05 ;Set V_LCD Range: upper range (PRS=1)
	rcall	i2c_send
	ldi	temp, 0x10 ;Set HV Gen Stages: 2x multiplication
;	ldi	temp, 0x11 ;Set HV Gen Stages: 3x multiplication
	rcall	i2c_send
	ldi	temp, 0x0C ;Display Control: LCD on, Inverted Video off
	rcall	i2c_send
	ldi	temp, 0x40 + 0 ;Set Y Address = 0
	rcall	i2c_send
	ldi	temp, 0x80 + 0 ;Set X Address = 0
	rcall	i2c_send
	rcall	i2c_stop
	ret
	
;--------------------

i2c_send:
	;send one byte via I2C
	ldi	temp2, 8
i2c_send_loop:
	sbrc	temp, 7
	cbi	LCD_DDR, LCD_SDA ;data = high
	sbrs	temp, 7
	sbi	LCD_DDR, LCD_SDA ;data = low
	lsl	temp
	rcall	i2c_delay
	cbi	LCD_DDR, LCD_SCL ;clock = high
	rcall	i2c_delay
	sbi	LCD_DDR, LCD_SCL ;clock = low
	rcall	i2c_delay
	dec	temp2
	brne	i2c_send_loop
	;read ACK
	cbi	LCD_DDR, LCD_SDA ;data = high
	rcall	i2c_delay
	cbi	LCD_DDR, LCD_SCL ;clock = high
	rcall	i2c_delay
	sbis	LCD_PIN, LCD_SDA ;skip if data = high (NACK)
	ldi	temp, 1
	sbi	LCD_DDR, LCD_SCL ;clock = low
	rcall	i2c_delay
	ret

;--------------------

i2c_start:
	;send I2C start condition
	sbi	LCD_DDR, LCD_SDA ;data = low
	rcall	i2c_delay
	sbi	LCD_DDR, LCD_SCL ;clock = low
	rcall	i2c_delay
	ret
	
;--------------------

i2c_stop:
	;send I2C stop condition
	sbi	LCD_DDR, LCD_SDA ;data = low
	rcall	i2c_delay
	cbi	LCD_DDR, LCD_SCL ;clock = high
	rcall	i2c_delay
	cbi	LCD_DDR, LCD_SDA ;data = high
	rcall	i2c_delay
	ret

;--------------------

i2c_delay:
	;delay for I2C transfers: rcall + ret = 7 cycles (7 us @ 1 MHz)
	ret
	